package org.opengroup.osdu.notification.provider.azure.messageBus.thread;

import com.google.common.base.Strings;
import com.google.gson.Gson;
import org.apache.commons.lang3.StringUtils;
import org.opengroup.osdu.core.common.cryptographic.HmacData;
import org.opengroup.osdu.core.common.cryptographic.ISignatureService;
import org.opengroup.osdu.core.common.cryptographic.SignatureServiceException;
import org.springframework.boot.autoconfigure.condition.ConditionalOnExpression;
import org.springframework.context.annotation.Primary;
import org.springframework.context.annotation.Scope;
import org.springframework.context.annotation.ScopedProxyMode;
import org.springframework.stereotype.Component;

import javax.crypto.Mac;
import javax.crypto.spec.SecretKeySpec;
import javax.xml.bind.DatatypeConverter;
import java.nio.charset.StandardCharsets;
import java.security.SecureRandom;
import java.util.Base64;

@Component
@Scope(value = "ThreadScope", proxyMode = ScopedProxyMode.TARGET_CLASS)
@ConditionalOnExpression("${azure.serviceBus.enabled:true} || ${azure.eventGridToServiceBus.enabled:true}")
@Primary
public class ThreadSignatureService implements ISignatureService {
    private static final String HMAC_SHA_256 = "HmacSHA256";
    private static final String DATA_FORMAT = "{\"expireMillisecond\": \"%s\",\"hashMechanism\": \"hmacSHA256\",\"endpointUrl\": \"%s\",\"nonce\": \"%s\"}";
    private static final String NOTIFICATION_SERVICE = "de-notification-service";
    private static final long EXPIRE_DURATION = 30000L;
    private static final String INVALID_SIGNATURE = "Invalid signature";
    private static final String ERROR_GENERATING_SIGNATURE = "Error generating the signature";
    private static final String SIGNATURE_EXPIRED = "Signature is expired";
    private static final String MISSING_HMAC_SIGNATURE = "HMAC signature should not be null or empty";
    private static final String MISSING_SECRET_VALUE = "Secret should not be null or empty";
    private static final String MISSING_ATTRIBUTES_IN_SIGNATURE = "Missing url or nonce or expire time in the signature";

    public ThreadSignatureService() {
    }

    public String getSignedSignature(String url, String secret) throws SignatureServiceException {
        if (!Strings.isNullOrEmpty(url) && !Strings.isNullOrEmpty(secret)) {
            long currentTime = System.currentTimeMillis();
            String expireTime = String.valueOf(currentTime + 30000L);
            String timeStamp = String.valueOf(currentTime);

            try {
                String nonce = DatatypeConverter.printHexBinary(this.generateRandomBytes(16)).toLowerCase();
                String data = String.format("{\"expireMillisecond\": \"%s\",\"hashMechanism\": \"hmacSHA256\",\"endpointUrl\": \"%s\",\"nonce\": \"%s\"}", expireTime, url, nonce);
                byte[] signature = this.getSignature(secret, nonce, timeStamp, data);
                byte[] dataBytes = data.getBytes(StandardCharsets.UTF_8);
                String dataBytesEncoded = Base64.getEncoder().encodeToString(dataBytes);
                StringBuilder output = new StringBuilder();
                output.append(dataBytesEncoded).append(".").append(DatatypeConverter.printHexBinary(signature).toLowerCase());
                return output.toString();
            } catch (Exception var13) {
                throw new SignatureServiceException("Error generating the signature", var13);
            }
        } else {
            throw new SignatureServiceException("Error generating the signature");
        }
    }

    public String getSignedSignature(String url, String secret, String expireTime, String nonce) throws SignatureServiceException {
        if (!Strings.isNullOrEmpty(url) && !Strings.isNullOrEmpty(secret) && StringUtils.isNumeric(expireTime)) {
            long expiry = Long.parseLong(expireTime);
            if (System.currentTimeMillis() > expiry) {
                throw new SignatureServiceException("Signature is expired");
            } else {
                String timeStamp = String.valueOf(expiry - 30000L);
                String data = String.format("{\"expireMillisecond\": \"%s\",\"hashMechanism\": \"hmacSHA256\",\"endpointUrl\": \"%s\",\"nonce\": \"%s\"}", expireTime, url, nonce);

                try {
                    byte[] signature = this.getSignature(secret, nonce, timeStamp, data);
                    return DatatypeConverter.printHexBinary(signature).toLowerCase();
                } catch (Exception var10) {
                    throw new SignatureServiceException("Error generating the signature", var10);
                }
            }
        } else {
            throw new SignatureServiceException("Error generating the signature");
        }
    }

    public void verifyHmacSignature(String hmac, String secret) throws SignatureServiceException {
        if (Strings.isNullOrEmpty(hmac)) {
            throw new SignatureServiceException("HMAC signature should not be null or empty");
        } else if (Strings.isNullOrEmpty(secret)) {
            throw new SignatureServiceException("Secret should not be null or empty");
        } else {
            String[] tokens = hmac.split("\\.");
            if (tokens.length != 2) {
                throw new SignatureServiceException("Invalid signature");
            } else {
                byte[] dataBytes = Base64.getDecoder().decode(tokens[0]);
                String requestSignature = tokens[1];
                String data = new String(dataBytes, StandardCharsets.UTF_8);
                HmacData hmacData = (HmacData)(new Gson()).fromJson(data, HmacData.class);
                String url = hmacData.getEndpointUrl();
                String nonce = hmacData.getNonce();
                String expireTime = hmacData.getExpireMillisecond();
                if (!Strings.isNullOrEmpty(url) && !Strings.isNullOrEmpty(nonce) && !Strings.isNullOrEmpty(expireTime)) {
                    String newSignature = this.getSignedSignature(url, secret, expireTime, nonce);
                    if (!requestSignature.equalsIgnoreCase(newSignature)) {
                        throw new SignatureServiceException("Invalid signature");
                    }
                } else {
                    throw new SignatureServiceException("Missing url or nonce or expire time in the signature");
                }
            }
        }
    }

    private byte[] getSignature(String secret, String nonce, String timeStamp, String data) throws Exception {
        byte[] secretBytes = DatatypeConverter.parseHexBinary(secret);
        byte[] nonceBytes = DatatypeConverter.parseHexBinary(nonce);
        byte[] encryptedNonce = this.computeHmacSha256(nonceBytes, secretBytes);
        byte[] encryptedTimestamp = this.computeHmacSha256(timeStamp, encryptedNonce);
        byte[] signedKey = this.computeHmacSha256("de-notification-service", encryptedTimestamp);
        byte[] signature = this.computeHmacSha256(data, signedKey);
        return signature;
    }

    private byte[] computeHmacSha256(String data, byte[] key) throws Exception {
        Mac mac = Mac.getInstance("HmacSHA256");
        mac.init(new SecretKeySpec(key, "HmacSHA256"));
        return mac.doFinal(data.getBytes(StandardCharsets.UTF_8));
    }

    private byte[] computeHmacSha256(byte[] data, byte[] key) throws Exception {
        Mac mac = Mac.getInstance("HmacSHA256");
        mac.init(new SecretKeySpec(key, "HmacSHA256"));
        return mac.doFinal(data);
    }

    private byte[] generateRandomBytes(int size) {
        byte[] key = new byte[size];
        SecureRandom secureRandom = new SecureRandom();
        secureRandom.nextBytes(key);
        return key;
    }
}